pub struct StandardContextHandler;Expand description
Standard context handler
Provides utilities for creating and managing execution contexts. This is a stateless handler that follows Layer 3 principles.
Implementations
Sourceimpl StandardContextHandler
impl StandardContextHandler
Sourcepub fn create_effect_context(
&self,
authority_id: AuthorityId,
context_id: ContextId,
execution_mode: ExecutionMode,
) -> EffectContext
pub fn create_effect_context( &self, authority_id: AuthorityId, context_id: ContextId, execution_mode: ExecutionMode, ) -> EffectContext
Create an effect context.
Convenience for creating a context with only an authority.
Sourcepub fn validate_context(
&self,
context: &EffectContext,
required_fields: &[&str],
) -> bool
pub fn validate_context( &self, context: &EffectContext, required_fields: &[&str], ) -> bool
Validate context for required metadata fields.
Sourcepub fn merge_metadata(
&self,
contexts: &[&EffectContext],
) -> HashMap<String, String>
pub fn merge_metadata( &self, contexts: &[&EffectContext], ) -> HashMap<String, String>
Merge metadata from multiple contexts.