pub struct NetworkMonitorHandler<E> { /* private fields */ }Expand description
Network monitor wrapper that debounces unusable transitions.
Implementations
Sourceimpl<E> NetworkMonitorHandler<E>
impl<E> NetworkMonitorHandler<E>
Sourcepub fn new(upstream: E) -> Self
pub fn new(upstream: E) -> Self
Create a monitor with the default unusable debounce duration (2 seconds).
Sourcepub fn with_debounce(upstream: E, unusable_debounce: Duration) -> Self
pub fn with_debounce(upstream: E, unusable_debounce: Duration) -> Self
Create a monitor with a custom unusable debounce duration.