RealSecureStorageHandler

Struct RealSecureStorageHandler 

pub struct RealSecureStorageHandler { /* private fields */ }
Real secure storage handler for production use

Implementations

Source

impl RealSecureStorageHandler

pub fn with_base_path(base_path: PathBuf) -> Self

Create a secure storage handler with a custom base path.

The secure storage files will be placed in base_path/secure_store/.