pub struct FilesystemStorageHandler { /* private fields */ }Expand description
Filesystem-based storage handler for production use
This handler stores data as files on the local filesystem. It is stateless and delegates all storage operations to the filesystem.
Implementations