pub struct MonitoringSystemHandler { /* private fields */ }Expand description
Stateless monitoring handler.
Implementations
Sourceimpl MonitoringSystemHandler
impl MonitoringSystemHandler
Sourcepub fn new(config: MonitoringConfig) -> Self
pub fn new(config: MonitoringConfig) -> Self
Create a new monitoring system handler
Sourcepub fn with_defaults() -> Self
pub fn with_defaults() -> Self
Create a monitoring system handler with default configuration
Sourcepub async fn check_component_health(
&self,
component: ComponentId,
) -> Result<HealthCheckResult, SystemError>
pub async fn check_component_health( &self, component: ComponentId, ) -> Result<HealthCheckResult, SystemError>
Manually trigger a health check for a specific component
Sourcepub async fn send_alert(
&self,
component: ComponentId,
severity: AlertSeverity,
title: &str,
message: &str,
metadata: HashMap<String, String>,
) -> Result<(), SystemError>
pub async fn send_alert( &self, component: ComponentId, severity: AlertSeverity, title: &str, message: &str, metadata: HashMap<String, String>, ) -> Result<(), SystemError>
Send a custom alert
Sourcepub async fn get_recent_alerts(&self, count: usize) -> Vec<Alert>
pub async fn get_recent_alerts(&self, count: usize) -> Vec<Alert>
Get the most recent alerts up to the specified count
Sourcepub async fn resolve_alert(&self, alert_id: u64) -> Result<(), SystemError>
pub async fn resolve_alert(&self, alert_id: u64) -> Result<(), SystemError>
Resolve an alert by its ID
Sourcepub async fn get_statistics(&self) -> MonitoringStats
pub async fn get_statistics(&self) -> MonitoringStats
Get monitoring statistics (stateless - delegates to external service)